package be.ggj.elektrofried.game;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;

import org.jdesktop.animation.timing.Animator;
import org.jdesktop.animation.timing.Animator.RepeatBehavior;
import org.jdesktop.animation.timing.TimingTargetAdapter;
import org.jdesktop.swingx.painter.Painter;

import be.ggj.elektrofried.ElektroFriedApp;
import be.ggj.elektrofried.sprites.Cloud;
import be.ggj.elektrofried.sprites.Elektron;
import be.ggj.elektrofried.sprites.ElektronCloud;
import be.ggj.elektrofried.sprites.Particle;
import be.ggj.elektrofried.sprites.Sprite;
import be.ggj.elektrofried.sprites.SteerableElektron;

import com.google.common.collect.ImmutableList;

public class ElektroFriedGame extends JPanel implements MouseListener {

	public static final String STEERABLE_ELEKTRON = "steerableElektron";
	public static final int DEFAULT_ANIMATION_DURATION = 1000000;
	public static final int SPEED_MULTIPLIER = 40;
	public static final double DEFAULT_SPEED = 0.11;
	public static final double DEFAULT_SPEED_INCREASE = 0.02;
	public static final double DEFAULT_ORDBIT_RADIUS = 400;

	private int level = 0;

	private List<Sprite<?>> spriteList = new ArrayList<Sprite<?>>();
	private List<Animator> animators = new ArrayList<Animator>();

	private Particle mainProton;
	private ElektronCloud elektronCloud;
	private SteerableElektron steerableElektron;
	private boolean collided = false;;

	public static final String LEVEL = "level";

	/**
	 * Initialize the particles using the level to set their parameters
	 * @param level the starting game level
	 * @param centerY the main particle's & panel x center coordinate
	 * @param centerX the main particle's & panel y center coordinate
	 */
	public ElektroFriedGame(int level) {
		addMouseListener(this);
	}

	/**
	 * Clear sprite list and recreate all particles using the given parameters
	 * @param centerX The center X coordinate of the main particle
	 * @param centerY The center Y coordinate of the main particle
	 * @param amount The number of elektrons in the cloud
	 * @param defaultSpeed The default speed for the elektrons in the cloud
	 */
	private void createParticles(int amount, double defaultSpeed) {
		Dimension size =  Toolkit.getDefaultToolkit().getScreenSize();
		int centerX = size.width / 2;
		int centerY = (size.height - 100) / 2;
		mainProton = createMainParticle(centerX, centerY, 25);
		elektronCloud = createElektronCloud(centerX, centerY, amount, defaultSpeed);
		SteerableElektron newSteerableElektron = createSteerableElektron(centerX, centerY, defaultSpeed, 10);
		firePropertyChange(STEERABLE_ELEKTRON, this.steerableElektron, this.steerableElektron = newSteerableElektron);
	}

	private SteerableElektron createSteerableElektron(int centerX, int centerY, double speed, int radius) {
		final SteerableElektron result = new SteerableElektron(centerX, centerY, DEFAULT_ORDBIT_RADIUS, DEFAULT_ORDBIT_RADIUS, speed, radius);
		Animator flyingElektronAnimator = new Animator(DEFAULT_ANIMATION_DURATION, new TimingTargetAdapter() {

			@Override
			public void timingEvent(float fraction) {
				float alpha = fraction * 360;
				double difference = result.getTargetRadius() > result.getOrbitRadiusX() ? 1 : -1;
				double variableOrbit = result.getOrbitRadiusX() * 0.95 + difference * 0.05 * result.getTargetRadius();
				result.setCoordinates(alpha, variableOrbit);
				if (mainProton.checkCollision(result)) {
					System.out.println("FINISH");
					for (Animator animator : getAnimators()) {
						animator.stop();
					}
					startNextLevel();
				}
				if (elektronCloud.checkCollision(result)) {
					collided = true;
					ElektroFriedApp.getApplication().shock(getLevel());
					result.changeTargetRadius(10);
				}
				repaint();
			}

		});
		flyingElektronAnimator.setResolution(10);
		flyingElektronAnimator.setRepeatBehavior(RepeatBehavior.LOOP);
		flyingElektronAnimator.setRepeatCount(Integer.MAX_VALUE);
		
		//flyingElektronAnimator.setAcceleration(0.2f);
		//flyingElektronAnimator.setDeceleration(0.2f);
		addAnimator(flyingElektronAnimator);
		addSprite(result);
		return result;
	}

	private Particle createMainParticle(int centerX, int centerY, int i) {
		final Particle result = new Particle(centerX, centerY, i);
		addSprite(result);
		return result;
	}

	private ElektronCloud createElektronCloud(int centerX, int centerY, int amount, double defaultSpeed) {
		final ElektronCloud result = new ElektronCloud(centerX, centerY, amount, defaultSpeed);
		Animator elektronAnimator = new Animator(DEFAULT_ANIMATION_DURATION, new TimingTargetAdapter() {

			public void timingEvent(float fraction) {
				double alpha = fraction * 360;
				for(Particle elektron : result.getParticles()) {
					//FIXME oh no a dirty cast!!
					((Elektron) elektron).setCoordinates(alpha);
				}
				repaint();
			}
			
		});
		elektronAnimator.setResolution(10);
		elektronAnimator.setRepeatBehavior(RepeatBehavior.LOOP);
		elektronAnimator.setRepeatCount(Integer.MAX_VALUE);
		//elektronAnimator.setStartFraction(0.2f);
		//elektronAnimator.setAcceleration(0.2f);
		//elektronAnimator.setDeceleration(0.2f);
		// start animation when glasspane is still visible
		elektronAnimator.start();
		addAnimator(elektronAnimator);
		addSprite(result);
		return result;
	}

	public void addElectron() {
		// TODO add elektron with the speed corresponding to the current level
		elektronCloud.addElektron(Color.orange);
	}

	public void startNextLevel() {
		int amount;
		double speed;
		if (elektronCloud != null) {
			amount = elektronCloud.getParticleAmount() + 1;
			speed = elektronCloud.getSpeed() + DEFAULT_SPEED_INCREASE;
		} else {
			amount = 1;
			speed = DEFAULT_SPEED;
		}
		setLevel(level + 1);
		spriteList.clear();
		animators.clear();
		createParticles(amount, speed);
	}
	
	private void setLevel(int level) {
		firePropertyChange(LEVEL, this.level, this.level = level );
	}

	private void addSprite(Sprite<?> sprite) {
		spriteList.add(sprite);
	}

	private void addAnimator(Animator animator) {
		animators.add(animator);
	}

	public int getLevel() {
		return level;
	}

	private <T extends Sprite<T>> List<T> getSpriteList(Class<T> filterClass) {
		ImmutableList.Builder<T> result = ImmutableList.builder();
		for (Sprite<?> sprite : spriteList) {
			if (filterClass.isAssignableFrom(sprite.getClass())) {
				result.add(filterClass.cast(sprite));
			}
		}
		return result.build();
	}

	public void setSpriteList(List<Sprite<?>> spriteList) {
		this.spriteList = spriteList;
	}

	@Override
	protected void paintComponent(Graphics arg0) {
		Graphics2D gfx2d = (Graphics2D) arg0;
		arg0.setColor(collided  ? Color.black : Color.white);
		collided = false;
		arg0.fillRect(0, 0, getSize().width, getSize().height);
		gfx2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		for (Particle particle : getSpriteList(Particle.class) ) {
			Painter<Particle> particlePainter = particle.getPainter();
			particlePainter.paint(gfx2d, particle, (int) particle.getX1(), (int) particle.getY1());
		}
		for (Cloud cloud : getSpriteList(Cloud.class)) {
			Painter<Cloud> cloudPainter = cloud.getPainter();
			cloudPainter.paint(gfx2d, cloud, (int) cloud.getX1(), (int) cloud.getY1());
		}
	}

	public List<Animator> getAnimators() {
		return animators;
	}

	@Override
	public void mouseClicked(MouseEvent arg0) {
//		if (SwingUtilities.isLeftMouseButton(arg0)) {
//			steerableElektron.changeTargetRadius(3);
//		} else if (SwingUtilities.isRightMouseButton(arg0)) {
//			steerableElektron.changeTargetRadius(-3);
//		}
	}


	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	boolean isPressed = true;

	@Override
	public void mousePressed(final MouseEvent arg0) {
		isPressed = true;
		new Thread(new Runnable() {

			@Override
			public void run() {
				while (isPressed) {
					if (SwingUtilities.isLeftMouseButton(arg0)) {
						steerableElektron.changeTargetRadius(2);
					} else if (SwingUtilities.isRightMouseButton(arg0)) {
						steerableElektron.changeTargetRadius(-2);
					}
					try {
						Thread.sleep(300);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
			
		}).start();
		
	}


	@Override
	public void mouseReleased(MouseEvent arg0) {
		isPressed = false;
	}



}
