package be.ggj.elektrofried.game;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

public class ElektroFriedGlassPane extends JComponent implements PropertyChangeListener {
	
	private int level = 1;
	
	public ElektroFriedGlassPane() {
		setOpaque(false);
		addMouseListener(new MouseAdapter() {

			@Override
			public void mouseClicked(MouseEvent e) {
				setVisible(false);
			}
			
		});
	}

	public int getLevel() {
		return level;
	}

	public void setLevel(int level) {
		this.level = level;
	}

	@Override
	protected void paintComponent(Graphics g) {
		// gets the current clipping area
		Rectangle clip = g.getClipBounds();
		// sets a 65% translucent composite
		AlphaComposite alpha = AlphaComposite.SrcOver.derive(0.50f);
		Graphics2D g2d = (Graphics2D) g;
		g2d.setComposite(alpha);
		// fills the background
		g.setColor(new Color(50, 50, 50));
		g.fillRect(clip.x, clip.y, clip.width, clip.height);
		// computes x and y, draws the text
		g2d.setFont(g2d.getFont().deriveFont(36f));
		g2d.setColor(Color.black);
		drawString(g2d, "Level " + getLevel(), clip, 350);
		g2d.setFont(g2d.getFont().deriveFont(16f));
		drawString(g2d, "Hitting an electron will make our electrofried device give you some electrical shocks.", clip, 200);
		drawString(g2d, "Bring the blue steerable electron to the proton without hitting any other electrons in the cloud.", clip, 250);
	}
	
	private void drawString(Graphics2D g2d, String string, Rectangle clip, int clipYOffset) {
		int messageWidth = SwingUtilities.computeStringWidth(g2d.getFontMetrics(), string);
		g2d.drawString(string,( clip.width - messageWidth) / 2, (clip.height - clipYOffset ) /2);
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(ElektroFriedGame.LEVEL)) {
			setLevel((Integer) evt.getNewValue());
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			setVisible(true);
		}
	}
	
}
